from risc_generator import Protections, FPQ1Contract, Premium
from risc_generator.content import fr, en
from tests.domain.contract.contract_base import ContractTestBase

INCLUDE = "include"
EXCLUDE = "exclude"


class ContractDeclarationTest(ContractTestBase):
    def setUp(self) -> None:
        self.a_protections = Protections(protections=self.a_dict_of_protections)
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        self.a_premium = Premium(**protection_premiums)

    def test_givenAFRContract_whenGenerateItem1Text_thenReturnProperFRText(self):
        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_1_text(
            insuree_name=fpq1_contract.insuree.name, insuree_address=fpq1_contract.insuree.address
        )

        a_insuree_name = "A Name"
        a_insuree_address = "An address\nA city, QC, A1A 1A1"

        expected = (
            "Article 1. NOM ET ADRESSE DE L'ASSURÉ DÉSIGNÉ*\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            "*La ville et la province de l'adresse écrite à cet article 1 constituent les lieux d'usage principal, "
            "de remisage et de stationnement du véhicule désigné. Si ce n'est pas le cas, le preneur ou l'assuré "
            "désigné doit le déclarer.\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContract_whenGenerateItem1Text_thenReturnProperENText(self):
        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_1_text(
            insuree_name=fpq1_contract.insuree.name, insuree_address=fpq1_contract.insuree.address
        )

        a_insuree_name = "A Name"
        a_insuree_address = "An address\nA city, QC, A1A 1A1"

        expected = (
            "Item 1. NAME AND ADDRESS OF THE NAMED INSURED*\n"
            f"{a_insuree_name}\n"
            f"{a_insuree_address}\n"
            "*The described vehicle is and will be mainly used, stored and parked in the town/city and "
            "province shown in Item 1. If not, the client or the named insured must so declare.\n"
        )

        self.assertEqual(expected, actual)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq13_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 13c\n"
        "Limitation under Protection 3 for vehicle glass\n"
        "(Section B)\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement limits coverage under Protection 3, Section B of the insurance contract by excluding"
        "damage caused to the glass of the specified vehicle, except in the case of:\n"
        "\t attempted theft;\n"
        "\t civil commotion;\n"
        "\t earthquakes;\n"
        "\t explosions;\n"
        "\t falling or forced landing of aircraft or parts of aircraft;\n"
        "\t fire;\n"
        "\t hail;\n"
        "\t lightning;\n"
        "\t riot;\n"
        "\t rising water;\n"
        "\t stranding, sinking, burning, derailment or collision of any vehicle or vessel "
        "in or upon which the specified vehicle is being transported;\n"
        "\t theft; and\n"
        "\t windstorms.\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from unittest import TestCase
from unittest.mock import ANY

from risc_generator import property_coverage_faker

INCLUDE = "include"
EXCLUDE = "exclude"


class PropertyCoverageFakerTests(TestCase):
    def assertEqualDict(self, expected, actual):
        for actual_key, actual_value in actual.items():
            expected_value = expected.get(actual_key)
            self.assertEqual(expected_value, actual_value)

    def test_givenB1Coverage_whenPropertyCoverageFaker_thenReturnIncludeAllOtherProtections(
        self,
    ):
        include_b1_coverage = True
        include_b2_coverage = False
        include_b3_coverage = False
        include_b4_coverage = False

        actual = property_coverage_faker(
            include_b1_coverage,
            include_b2_coverage,
            include_b3_coverage,
            include_b4_coverage,
        )

        expected = {
            "PropertyDamageB1Coverage": ANY,
            "PropertyDamageB2Coverage": INCLUDE,
            "PropertyDamageB3Coverage": INCLUDE,
            "PropertyDamageB4Coverage": INCLUDE,
        }

        self.assertEqualDict(expected=expected, actual=actual)

    def test_givenB2OnlyCoverage_whenPropertyCoverageFaker_thenReturnExcludeAllOtherProtections(
        self,
    ):
        include_b1_coverage = False
        include_b2_coverage = True
        include_b3_coverage = False
        include_b4_coverage = False

        actual = property_coverage_faker(
            include_b1_coverage,
            include_b2_coverage,
            include_b3_coverage,
            include_b4_coverage,
        )

        expected = {
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": ANY,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": EXCLUDE,
        }

        self.assertEqualDict(expected=expected, actual=actual)

    def test_givenB3OnlyCoverage_whenPropertyCoverageFaker_thenReturnExcludeAllOtherProtections(
        self,
    ):
        include_b1_coverage = False
        include_b2_coverage = False
        include_b3_coverage = True
        include_b4_coverage = False

        actual = property_coverage_faker(
            include_b1_coverage,
            include_b2_coverage,
            include_b3_coverage,
            include_b4_coverage,
        )

        expected = {
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": EXCLUDE,
            "PropertyDamageB3Coverage": ANY,
            "PropertyDamageB4Coverage": INCLUDE,
        }

        self.assertEqualDict(expected=expected, actual=actual)

    def test_givenB4OnlyCoverage_whenPropertyCoverageFaker_thenReturnExcludeAllOtherProtections(
        self,
    ):
        include_b1_coverage = False
        include_b2_coverage = False
        include_b3_coverage = False
        include_b4_coverage = True

        actual = property_coverage_faker(
            include_b1_coverage,
            include_b2_coverage,
            include_b3_coverage,
            include_b4_coverage,
        )

        expected = {
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": EXCLUDE,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": ANY,
        }

        self.assertEqualDict(expected=expected, actual=actual)

    def test_givenB2AndB3OnlyCoverage_whenPropertyCoverageFaker_thenReturnExcludeAllOtherProtections(
        self,
    ):
        include_b1_coverage = False
        include_b2_coverage = True
        include_b3_coverage = True
        include_b4_coverage = False

        actual = property_coverage_faker(
            include_b1_coverage,
            include_b2_coverage,
            include_b3_coverage,
            include_b4_coverage,
        )

        expected = {
            "PropertyDamageB1Coverage": INCLUDE,
            "PropertyDamageB2Coverage": ANY,
            "PropertyDamageB3Coverage": ANY,
            "PropertyDamageB4Coverage": INCLUDE,
        }

        self.assertEqualDict(expected=expected, actual=actual)

    def test_givenB2AndB4OnlyCoverage_whenPropertyCoverageFaker_thenReturnExcludeAllOtherProtections(
        self,
    ):
        include_b1_coverage = False
        include_b2_coverage = True
        include_b3_coverage = False
        include_b4_coverage = True

        actual = property_coverage_faker(
            include_b1_coverage,
            include_b2_coverage,
            include_b3_coverage,
            include_b4_coverage,
        )

        expected = {
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": ANY,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": ANY,
        }

        self.assertEqualDict(expected=expected, actual=actual)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq16_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 16\n"
        "Suspension of coverage during vehicle storage\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract. "
        "Details required for the endorsement may be entered in the ''Declarations'' section or in "
        "the endorsement itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N: \n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Date of storage:  at 12:01 A.M. standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement suspends certain coverage under the insurance contract due "
        "to storage of the specified vehicle.\n"
        "Suspension of coverage\n"
        "The named insured declares that the specified vehicle will be withdrawn from use "
        "and stored as of the date of storage and therefore requests that the following coverage "
        "be suspended from that date:\n"
        "1. Suspended coverage under Section A\n"
        "Coverage under Section A of the insurance contract is suspended with regard to the "
        "driving or use of the following vehicles:\n"
        "\t specified vehicle;\n"
        "\t any vehicle of which the named insured has recently become the owner that "
        "replaces or is in addition to the specified vehicle;\n"
        "\t any temporary substitute vehicle that replaces the specified vehicle.\n"
        "The named insured will continue to benefit from the other coverage under Section A.\n"
        "2. Suspended coverage under Section B\n"
        "Coverage under Protections 1 and 2, Section B of the insurance contract is suspended "
        "with regard to the driving or use of the following vehicles:\n"
        "\t specified vehicle;\n"
        "\t any vehicle of which the named insured has recently become the owner that "
        "replaces or is in addition to the specified vehicle;\n"
        "\t any temporary substitute vehicle that replaces the specified vehicle.\n"
        "The named insured will continue to benefit from the other coverage under Section B.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "End of suspension of coverage\n"
        "The suspended coverage will be reinstated on one of the following dates:\n"
        "\t on the date determined by the named insured, if he or she has first informed the "
        "insurer accordingly;\n"
        "\t if no date has thus been determined, on the April 1st following the date of storage.\n"
        "The insurance contract must be in force for coverage to be reinstated.\n"
        "Rebate\n"
        "The named insured is entitled to a rebate for the storage period, calculated on the following basis:"
        "\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



"""
The code below was copied from the Faker Vehicle project, and has been modified for the purpose of this package.

COPYRIGHT

All contributions from the https://github.com/kennethwsmith/faker_vehicle authors.
Copyright (c) 2020 - January 16 2023
All rights reserved.

Each contributor holds copyright over their respective contributions. The project versioning (Git)
records all such contribution source information.

LICENSE

The MIT License (MIT)

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES
OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Modification are:
 - removal of machine
 - removal of Category
"""

from random import choice

from faker.providers import BaseProvider

from .vehicle_dict import vehicles


class VehicleProvider(BaseProvider):
    """
    A Provider for vehicle related test data.

    >>> from faker import Faker
    >>> from faker_vehicle import VehicleProvider
    >>> fake = Faker()
    >>> fake.add_provider(VehicleProvider)
    """

    def vehicle_object(self):
        """
        Returns a random vehicle dict example:
        {"Year": 2008, "Make": "Jeep", "Model": "Wrangler", "Electric Vehicle": False}
        """
        veh = choice(vehicles)
        return veh

    def vehicle_year_make_model(self):
        """Returns Year Make Model example: 1997 Nissan 240SX"""
        veh = self.vehicle_object()
        year = veh.get('Year')
        make = veh.get('Make')
        model = veh.get('Model')
        return str(year) + ' ' + make + ' ' + model

    def vehicle_year_make_model_cat(self):
        """
        Returns Year Make Model Cat example:
        2017 GMC Sierra 1500 Double Cab (Pickup)
        """
        veh = self.vehicle_object()
        year = veh.get('Year')
        make = veh.get('Make')
        model = veh.get('Model')
        return str(year) + ' ' + make + ' ' + model

    def vehicle_make_model(self):
        """Returns Make Model example: Audi Q7"""
        veh = self.vehicle_object()
        make = veh.get('Make')
        model = veh.get('Model')
        return make + ' ' + model

    def vehicle_make(self):
        """Returns Make example: Lincoln"""
        veh = self.vehicle_object()
        return veh.get('Make')

    def vehicle_year(self):
        """Returns Year example: 1999"""
        veh = self.vehicle_object()
        return str(veh.get('Year'))

    def vehicle_model(self):
        """Returns Model example: Frontier King Cab"""
        veh = self.vehicle_object()
        return veh.get('Model')



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq31_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 31\n"
        "Equipment not owned by the named insured\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N: \n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement extends coverage under the insurance contract to the following "
        "equipment not owned by the named insured:\n"
        "Application\n"
        "The equipment must be normally attached to the specified vehicle. In addition, "
        "the named insured must have the care, custody or control of such equipment.\n"
        "1. Section A\n"
        "Coverage under Section A is extended to the financial consequences that an "
        "insured person may incur if held civilly liable for damage caused to another person "
        "and resulting from the equipment described above.\n"
        "2. Section B\n"
        "Coverage under Section B as applicable to the specified vehicle is extended to:\n"
        "\t direct and accidental damage caused to the equipment described above;\n"
        "\t the disappearance of such equipment.\n"
        "For Section B, the indemnity will be payable up to the ''actual cash value'' of the "
        "equipment, subject to a maximum of   $ Payment will be made jointly to the named "
        "insured and  , to the extent of their respective interests.\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



import random

import numpy as np
import torch
from faker import Faker

from ... import Insuree, Protections, Premium, change_to_snake_case


class QuebecPremiumFaker(Faker):
    def __init__(self) -> None:
        """
        The values of the mu uses are base on the 2021 GAA statistics
        (https://gaa.qc.ca/fr/statistiques/cout-de-l-assurance-automobile/cout-de-l-assurance-
        pour-les-voitures-de-tourisme/).
        The 2 millions mu bonus is base on our overall experience of the premium base increase (~2$/month usually).

        The value of the sigma, 0.5, is selected so the concentration the distribution around the mu value.
        """
        super().__init__()
        self.liability_mu = 351
        self.liability_mu_increase = 48

        self.sigma = 0.5

        self.property_damage_b2_coverage_mu = 328
        self.property_damage_b3_coverage_mu = 174
        self.property_damage_b1_coverage_mu = self.property_damage_b2_coverage_mu + self.property_damage_b3_coverage_mu
        self.property_damage_b4_coverage_mu = round(174 * 0.99)  # This protection is a little less that B3

        self.endorsement_20_mu = 50
        self.endorsement_33_mu = 40
        self.endorsement_34_mu = 10
        self.endorsement_43_mu = 250

    def premium(self, protections: Protections, insuree: Insuree) -> Premium:
        # pylint: disable=too-many-branches
        """
        Faker to generate a premium base on an insuree risk. By default, it uses Quebec province premium statistic.
        protections (Protections): Protection to generate the premium.
        insuree (Insuree): Insuree to base the risk factor on.

        About the probabilistic distribution parameters:

        Mu risk factors:
            - Age and Sex: Base on the 2021 GAA Report
                (https://gaa.qc.ca/fr/statistiques/criteres-de-tarification/selon-l-age-et-le-sexe/)
                equally averaged (for simplification).
            - Suspension: We arbitrarily set the multiplication factors to two (2), which reflect our overall
                experience of the premium multiplication factor in such case.
            - number of claims: We arbitrarily set the multiplication factors as the following:
                - if 1 claim: 8% more (base factor).
                - if 2 claim: two time the base factor.
                - if 3 claims or more: three time the base factor.
                It reflects our overall experience of the claims effect on the premium but greatly simplified since
                it does not take into account the claim type and amount.

        Return:
            A premium for the protections that generate an increase in the premium.
        """

        liability_mu = self.liability_mu
        if protections.get("LiabilityCoverage") == 2000000:
            # Increase liability mu if 2 million protection amount
            liability_mu += self.liability_mu_increase

        mu_risk_factor = 1
        if insuree.sex in ("male", "homme"):
            mu_risk_factor *= 1.1597
        if insuree.suspension:
            mu_risk_factor *= 2
        if insuree.number_of_claims_past_years == 1:
            mu_risk_factor *= 1.08
        elif insuree.number_of_claims_past_years == 2:
            mu_risk_factor *= 1.16
        elif insuree.number_of_claims_past_years >= 3:
            mu_risk_factor *= 1.32

        protections_premiums = {}

        liability_coverage_premium = round(random.gauss(mu=liability_mu * mu_risk_factor, sigma=self.sigma))
        protections_premiums.update({"LiabilityCoveragePremium": liability_coverage_premium})

        property_damage_protections = protections.property_damage_protections()
        for key in property_damage_protections:
            mu = getattr(self, f"{change_to_snake_case(key)}_mu")
            premium = round(random.gauss(mu=mu * mu_risk_factor, sigma=self.sigma))

            protections_premiums.update({f"{key}Premium": premium})

        if protections.is_protected("FAQ20Coverage"):
            premium = round(random.gauss(mu=self.endorsement_20_mu * mu_risk_factor, sigma=self.sigma))
            protections_premiums.update({"FAQ20CoveragePremium": premium})
        if protections.is_protected("FAQ20ACoverage"):
            premium = round(random.gauss(mu=self.endorsement_20_mu * mu_risk_factor, sigma=self.sigma))
            protections_premiums.update({"FAQ20ACoveragePremium": premium})
        if protections.is_protected("FAQ33Coverage"):
            premium = round(random.gauss(mu=self.endorsement_33_mu * mu_risk_factor, sigma=self.sigma))
            protections_premiums.update({"FAQ33CoveragePremium": premium})
        if protections.is_protected("FAQ34Coverage"):
            premium = round(random.gauss(mu=self.endorsement_34_mu * mu_risk_factor, sigma=self.sigma))
            protections_premiums.update({"FAQ34CoveragePremium": premium})
        if protections.is_protected("FAQ34ABCoverage"):
            premium = round(random.gauss(mu=self.endorsement_34_mu * mu_risk_factor, sigma=self.sigma))
            protections_premiums.update({"FAQ34ABCoveragePremium": premium})
        if protections.is_protected("FAQ43Coverage"):
            premium = round(random.gauss(mu=self.endorsement_43_mu * mu_risk_factor, sigma=self.sigma))
            protections_premiums.update({"FAQ43CoveragePremium": premium})

        return Premium(**protections_premiums)

    def seed_instance(self, seed=42) -> None:
        """
        seed (int): The seed value to use for the generation seed. By default, 42.
        """
        super().seed_instance(seed)
        random.seed(seed)
        np.random.seed(seed)
        torch.manual_seed(seed)



from unittest import TestCase

from risc_generator import liability_coverage_faker


class LiabilityCoverageFakerTests(TestCase):
    n = 250000
    delta = 0.05  # A delta of 0.05, meaning we accept a 5% deviation from the expected probabilities

    def test_givenASyntheticCoverage_whenNotOnlyLiabilityCoverage_thenAreInDeltaExpectedNotOnlyProb(
        self,
    ):
        coverages = [
            liability_coverage_faker(is_only_liability_coverage=False).get("LiabilityCoverage") == 1000000
            for _ in range(self.n)
        ]

        actual = sum(coverages) / self.n  # Prob of having 1 million in liability coverage
        expected = 0.6

        self.assertAlmostEqual(actual, expected, delta=self.delta)

    def test_givenASyntheticCoverage_whenNotOnlyLiabilityCoverage_thenAreInDeltaExpectedOnlyProb(
        self,
    ):
        coverages = [
            liability_coverage_faker(is_only_liability_coverage=True).get("LiabilityCoverage") == 1000000
            for _ in range(self.n)
        ]

        actual = sum(coverages) / self.n  # Prob of having 1 million in liability coverage
        expected = 0.95

        self.assertAlmostEqual(actual, expected, delta=self.delta)



from unittest import TestCase

from risc_generator import generate_a_client_number


class ContractFakerToolsTest(TestCase):
    def test_whenGenerateAClientNumber_thenIs6Digits(self):
        client_number = generate_a_client_number()
        actual = len(client_number)
        expected = 6
        self.assertEqual(expected, actual)

    def test_whenGenerateAClientNumber_thenIsDigits(self):
        client_number = generate_a_client_number()
        expected = str
        self.assertIsInstance(client_number, expected)



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq37ab_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 37 (A-B)\n"
        "Changes to coverage for electronic equipment\n"
        "(Section B)\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "Depending on the applicable option, this endorsement changes the coverage under Section B "
        "of the insurance contract for ''electronic equipment'' that is not ''original electronic equipment.''\n"
        "Option A: Limitation of indemnity\n"
        "In the event of a covered loss, the indemnity under Section B will be limited to  $ for "
        "all damaged ''electronic equipment'' that is not ''original electronic equipment.''\n"
        "In addition, the indemnity may not be greater than the ''actual cash value'' of the "
        "damaged ''electronic equipment''.\n"
        "This limitation will apply even when a Q.E.F. No. 43 (A to F) entitled ''Change to "
        "loss payment'' is attached to the insurance contract.\n"
        "Option B: Exclusion from coverage\n"
        "All ''electronic equipment'' that is not ''original electronic equipment'' will be excluded "
        "from coverage under Section B.\n"
        "Definitions\n"
        "1. Electronic equipment\n"
        "The expression ''electronic equipment'' as used in this endorsement means electronic devices:\n"
        "that are installed or meant to be installed permanently in or on the specified vehicle; and\n"
        "that are used for communicating, reproducing or recording sound or images or both "
        "simultaneously, including:\n"
        "\t - compact disc and DVD players;\n"
        "\t - personal navigation devices (GPS);\n"
        "\t - radios, including citizens band, two-way amateur or other radios; and\n"
        "\t - sound systems.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "2. Original electronic equipment\n"
        "The expression ''original electronic equipment'' as used in this endorsement means "
        "the electronic equipment:\n"
        "\t that is installed by the manufacturer or the dealer; and\n"
        "\t that was included in the original purchase price of the specified vehicle.\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq43_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 43 (A to F)\n"
        "Change to indemnity\n"
        "(Section B)\n"
        "The endorsement heading and the applicable options must be entered in the ''Declarations'' section of "
        "the insurance contract. Details required for the endorsement may be entered in the ''Declarations'' "
        "section or in the endorsement itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N: \n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Additional insurance premium payable:\n"
        "\t Amounts payable:  \n"
        "\t Due date:\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement extends coverage under Section B of the insurance contract by "
        "changing, depending on the applicable options, Article 2, ''Reporting a loss and "
        "submitting a claim.'' Changes made by these options relate to how the value of damage is calculated.\n"
        "APPLICATION OF ENDORSEMENT\n"
        "For the options to apply to the specified vehicle, they must be entered in the "
        "''Declarations'' section of the insurance contract.\n"
        "Except for Option 43C, the named insured may elect settlement in accordance "
        "with Article 2, ''Reporting a loss and submitting a claim'', without the selected "
        "option being applied. In such instance, the named insured must submit a request to the insurer.\n"
        "The deductible entered in the ''Declarations'' section for the specified vehicle will "
        "be assumed by the named insured.\n"
        "VALUE OF DAMAGE IN EVENT OF PARTIAL LOSS\n"
        "Options 43A and 43B relate to calculation of the value of damage payable by the "
        "insurer in the event of repair or replacement of damaged parts.\n"
        "These options change Article 2.1, ''Reporting a loss and submitting a claim.'' However, "
        "the rules set out in the first paragraph of Article 2.1.1 of that section continue to apply.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Option 43A – Partial loss – New parts\n"
        "In the event of partial loss, when the damaged parts can be repaired, the value of "
        "damage will be calculated in accordance with Article 2.1, ''Reporting a loss and "
        "submitting a claim.'' However, there will be no deduction for depreciation.\n"
        "If the insurer determines that damaged parts cannot be repaired and must be "
        "replaced, there will be no deduction for depreciation. In addition:\n"
        "\t the value of damage will be calculated on the basis of the cost of new "
        "original equipment manufacturer parts;\n"
        "\t if glass is damaged, the named insured may ask for it to be replaced by a new "
        "original equipment manufacturer part or by a new similar part.\n"
        "If a new part is out of stock or no longer being manufactured, the liability of the insurer will be "
        "limited to the latest list price of the new original equipment manufacturer part.\n"
        "If the damaged parts are not repaired or replaced by new parts, Option 43A will not apply.\n"
        "Option 43B – Partial loss – Waiver of depreciation\n"
        "In the event of partial loss, the value of damage will be calculated in accordance with Article 2.1, "
        "''Reporting a loss and submitting a claim.'' However, if the insurer determines that "
        "depreciation is to be applied to the value of damage, this option will cover "
        "such depreciation up to a maximum of   $ per loss.\n"
        "However, if the damaged parts are not repaired or replaced, Option 43B will not apply.\n"
        "VALUE OF DAMAGE IN EVENT OF TOTAL LOSS OR TOTAL CONSTRUCTIVE LOSS\n"
        "Options 43C, 43D, 43E and 43F relate to calculation of the value of "
        "damage payable by the insurer in the event of total loss or total constructive loss. "
        "They amend Article 2.2, ''Reporting a loss and submitting a claim.''\n"
        "Option 43C – Total loss – Value of vehicle agreed in advance\n"
        "In the event of total loss or total constructive loss, the named insured and the insurer "
        "agree that the value of the specified vehicle, on the date of the loss, will be:  $ (''agreed value'').\n"
        "Option43D – Total loss – Waiver of depreciation\n"
        "In the event of total loss or total constructive loss, the value of damage "
        "will be equal to the least of the following amounts:\n"
        "\t the price paid for the specified vehicle by the named insured who is the actual owner;\n"
        "\t the current price of the specified vehicle on the original date of purchase; or\n"
        "\t the price for which the insurer may, on the date of the loss, purchase a new "
        "vehicle with specifications or equipment and accessories similar to those of the specified vehicle.\n"
        "Option 43E – Total loss – Replacement cost\n"
        "In the event of total loss or total constructive loss, the named insured may choose one "
        "of the following three indemnity options:\n"
        "1. Replacement by a new vehicle\n"
        "If the specified vehicle is replaced by a new vehicle with the "
        "same specifications or equipment and accessories, the value of damage will be equal to the "
        "price of this vehicle.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        f"If no such vehicle is available and the specified vehicle is replaced by a "
        f"new vehicle with similar specifications or equipment and accessories, the value of damage "
        f"will be equal to the price of this vehicle.\n"
        f"2. Replacement by a used vehicle, or by a new vehicle other than as in paragraph 1\n"
        f"If the specified vehicle is replaced by a used vehicle, or by a new vehicle other than as in "
        f"paragraph 1, the value of damage will be equal to:\n"
        f"\t the price paid for the specified vehicle by the named insured who is the actual "
        f"owner; or\n"
        f"\t the price of the used or new vehicle that replaces the specified vehicle; whichever is higher.\n"
        f"However, the value of damage may not be greater than the price that the "
        f"insurer would have paid had paragraph 1 applied.\n"
        f"3. Non-replacement of specified vehicle\n"
        f"If the specified vehicle is not replaced, the value of damage will be equal to:\n"
        f"\t the price paid for the specified vehicle by the named insured who is the actual "
        f"owner; or\n"
        f"\t the current price of the specified vehicle on the original date of "
        f"purchase, whichever is lower.\n"
        f"However, the value of damage may not be greater than the price that the insurer "
        f"would have paid had paragraph 1 applied.\n"
        f"Option 43F – Total loss – Increased settlement\n"
        f"In the event of total loss or total constructive loss, the value of damage will be equal to:\n"
        f"\t A – the price paid for the specified vehicle by the named insured who is the actual owner, "
        f"increased by  % compounded annually and calculated "
        f"proportionally to the number of days elapsed between   (date) and the date of the loss;\n"
        f"or\n"
        f"B – the amount of  $ increased by  "
        f"compounded annually and calculated proportionally to the number of days elapsed "
        f"between  (date) and the date of the loss;\n"
        f"or\n"
        f"C – the ''actual cash value'' of the specified vehicle increased by  % "
        f"compounded annually and calculated proportionally to the number of days elapsed between "
        f" (date) and the date of the loss;\n"
        f"or\n"
        f"D – the ''actual cash value'' of the specified vehicle increased by  $.\n"
        f"In all instances, the value of damage may not be greater than the price for which "
        f"the insurer may, on the date of the loss, purchase a new vehicle with specifications, "
        f"equipment and accessories similar to those of the specified vehicle.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "SPECIFIC RULE FOR VEHICLE LEASED OR UNDER A CONTRACT OF LEASING\n"
        "If the specified vehicle is leased or under a contact of leasing, and if Option 43D, 43E or "
        "43F applies and the owner and lessee are designated in the insurance contract, only the "
        "lessee will be entitled to the difference between:\n"
        "\t the value of the specified vehicle, as calculated in accordance with the applicable option; and\n"
        "\t the ''actual cash value'' of the specified vehicle, as calculated in accordance "
        "with Article 2.2, ''Reporting a loss and submitting a claim.''\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq27a_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 27a\n"
        "Responsabilité civile du fait de dommages causés à des véhicules "
        "dont l'assuré désigné n'est pas propriétaire\n"
        "(excluant les véhicules fournis par un employeur)\n"
        "(Chapitre A)\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat "
        "d'assurance. Quant aux informations requises dans l'avenant, elles peuvent être écrites à "
        "cette section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N :\n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Date de fin :\n"
        "cet avenant s'applique jusqu'au  à 0 h 01, heure normale à "
        "l'adresse de l'assuré désigné ou, si aucune date n'est précisée, jusqu'à la date "
        "d'échéance du contrat d'assurance.\n"
        "Prime d'assurance additionnelle à payer :\n"
        "\t Montants à payer :  \n"
        "\t Date limite pour payer :  \n"
        "Description de l'avenant\n"
        "Cet avenant restreint les garanties du contrat d'assurance lorsque le véhicule visé est, au "
        "moment du sinistre, conduit ou utilisé par :  (nom du conducteur désigné)\n"
        "Dans un tel cas, seuls sont couverts les risques suivants, ou ceux écrits spécifiquement pour cet avenant "
        "à la section « Conditions particulières » du contrat d'assurance :.\n"
        "Cet avenant étend la garantie du chapitre A du contrat d'assurance aux conséquences financières que "
        "peut subir la personne assurée lorsqu'elle est civilement responsable du fait :\n"
        "\t d'un dommage causé à un véhicule assimilable à  ou à ses équipements et accessoires;\n"
        "\t de la disparition de ce véhicule ou de ses équipements et accessoires.\n"
        "La responsabilité civile peut être contractuelle ou extracontractuelle.\n"
        "Personnes assurées\n"
        "L'expression « personne assurée » utilisée dans cet avenant fait référence aux personnes suivantes :\n"
        "\t l'assuré désigné;\n"
        "\t son conjoint;\n"
        "\t toute personne désignée dans un F.A.Q. N 2 intitulé « Conduite de véhicules dont "
        "l'assuré désigné n'est pas propriétaire par des conducteurs désignés », annexé au contrat d'assurance;\n"
        "\t les représentants légaux et la succession de toutes les personnes assurées énumérées ci-dessus.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Par contre, lorsque l'assuré désigné est une personne morale, une société ou une association, "
        "l'expression « personne assurée » fait référence aux personnes suivantes :\n"
        "\t tout employé, actionnaire, associé ou membre autorisé par l'assuré désigné;\n"
        "\t leur conjoint;\n"
        "\t toute personne désignée dans un F.A.Q. N 2, annexé au contrat d'assurance;\n"
        "\t les représentants légaux et la succession de toutes les personnes assurées énumérées ci-dessus.\n"
        "Ou uniquement :\n"
        "\t aux personnes suivantes :  ;\n"
        "\t à leur conjoint;\n"
        "\t à toute personne désignée dans un F.A.Q. N 2, annexé au contrat d'assurance;\n"
        "\t aux représentants légaux et à la succession de toutes les personnes assurées énumérées ci-dessus.\n"
        "Conditions d'application\n"
        "1. La personne assurée doit avoir un pouvoir de direction ou de gestion sur le véhicule, "
        "ses équipements et accessoires, ou en avoir la garde.\n"
        "2. La personne assurée, ou toute personne ayant le même domicile que l'assuré désigné, "
        "ne doit pas être propriétaire du véhicule, ni en être locataire pour au moins un an ou crédit-preneur.\n"
        "3. Un employeur de la personne assurée ou un employeur d'une personne ayant le même domicile "
        "que l'assuré désigné ne doit pas être propriétaire du véhicule, ni en être locataire pour au moins "
        "un an ou crédit-preneur.\n"
        "Risques couverts et primes d'assurance\n"
        "Seuls sont couverts les risques pour lesquels une franchise ou une prime d'assurance est écrite au "
        "tableau ci-dessous, ou écrite spécifiquement pour cet avenant à la section « Conditions particulières » "
        "du contrat d'assurance :\n"
        "CHAPITRE A : Garantie pour la responsabilité civile découlant des dommages matériels et des "
        "dommages corporels causés à d'autres personnes\n"
        " RISQUES | FRANCHISES | PRIMES D'ASSURANCE\n"
        "Protection 1 : « Tous risques », Protection 2 : Risques de collision et de renversement, "
        "Protection 3 : Tous les risques sauf collision ou renversement, "
        "Protection 4 : Risques spécifiques | Franchise par sinistre : $ $ $ $ | $ $ $ $ \n"
        "  | Total : |  $"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Précisions\n"
        "1. Les Protections 1, 2, 3 et 4 ont la même signification qu'au chapitre B du contrat "
        "d'assurance. Les exclusions prévues à ce chapitre s'appliquent si le cas se présente.\n"
        "2. Un montant d'assurance de   $ s'applique par sinistre, auquel s'ajoutent les frais, "
        "les dépens et les intérêts qui découlent d'une poursuite.\n"
        "3. Si le cas se présente, les garanties additionnelles du chapitre A du contrat d'assurance "
        "peuvent s'appliquer.\n"
        "4. L'assureur s'engage à n'exercer aucun recours contre la personne qui a, avec le "
        "consentement de la personne assurée, un pouvoir de direction ou de gestion sur le véhicule ou "
        "ses équipements et accessoires, ou qui en a la garde, sauf si cette personne :\n"
        "\t exerçait une activité professionnelle de garagiste au moment du sinistre; ou\n"
        "\t n'a pas respecté le contrat d'assurance.\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from typing import Union


class Vehicle:
    def __init__(
        self, year: int, make: str, model: str, vin: str, purchase_condition: str, electric_vehicle: bool
    ) -> None:
        """
        A vehicle.

        year (int): The car year (e.g. 2020).
        make (str): The car maker (e.g. Honda).
        model (str): The model of the car (e.g. Civic).
        vin (str): The vin number of the car.
        purchase_condition (str): The purchase condition of the car in the locale language (e.g. 'used').
        electric_vehicle (bool): Either or not the car is an electric one.
        """
        self.year = year
        self.make = make
        self.model = model
        self.vin = vin
        self.purchase_condition = purchase_condition
        self.electric_vehicle = electric_vehicle

    def __str__(self) -> str:
        return (
            f"{self.make.capitalize()} {self.model.capitalize()} "
            f"{str(self.year).capitalize()} VIN: {self.vin.capitalize()}"
        )

    def is_electric(self) -> bool:
        """
        Method to get if vehicle is an electric one.
        """
        return self.electric_vehicle

    def format_vehicle_complete_details(self, with_text: Union[None, str] = None) -> str:
        """
        Method to format the vehicle complete details.

        with_text (Union[None, str]): Either or not to include wrapping details with the vehicle complete details. If
            `with_text=None`, the vehicle details will be formatted as the following:
            `'vehicle_make vehicle_model vehicle_year vehicle_vin'`. If `with_text='Text'`, the vehicle details will be
            formatted as the following: `'vehicle_make - vehicle_model - vehicle_year - "Text" vehicle_vin'`.
            By default, None.

        Return:
            The formatted details text.
        """
        if with_text is not None:
            text = [" - ", " - ", f" - {with_text} "]
        else:
            text = [" "] * 3
        return f"{self.make}{text[0]}{self.model}{text[1]}{str(self.year)}{text[2]}{self.vin}"

    def format_vehicle_make_year_details(self) -> str:
        """
        Method to format the make and year vehicle details.

        Return:
            The formatted make and year details.
        """
        return f"{self.make} - {str(self.year)}"

    def format_vehicle_make_model_year_details(self) -> str:
        """
        Method to format the make, model and year vehicle details.

        Return:
            The formatted make, model and year details.
        """
        return f"{self.make} {self.model} {str(self.year)}"



from typing import Dict, List, Union

from .tools import change_to_snake_case, change_to_camel_case

INCLUDE = "include"
EXCLUDE = "exclude"


class Protections:
    def __init__(self, protections: Dict) -> None:
        for key, value in protections.items():
            # We save attribute in snake case format and use the same function in a getter method
            # to get the proper attribute using the CamelCase coverage format.
            self.__setattr__(change_to_snake_case(key), value)

    def __str__(self) -> str:
        chapter_a_text = f"Chapter A: {self.get('LiabilityCoverage')}$\n"

        chapter_b_text = "Chapter B: \n"

        # Not in include or exclude since we want the case with the franchise amount

        b1 = self.get("PropertyDamageB1Coverage")
        if b1 is not None and b1 not in (INCLUDE, EXCLUDE):
            chapter_b_text += f"All risk protected with {b1}$ franchise\n"

        b2 = self.get("PropertyDamageB2Coverage")
        if b2 is not None and b2 not in (INCLUDE, EXCLUDE):
            chapter_b_text += f"Collision protected with {b2}$ franchise\n"

        b3 = self.get("PropertyDamageB3Coverage")
        if b3 is not None and b3 not in (INCLUDE, EXCLUDE):
            chapter_b_text += f"Other non collision protected with {b3}$ franchise\n"

        b4 = self.get("PropertyDamageB4Coverage")
        if b4 is not None and b4 not in (INCLUDE, EXCLUDE):
            chapter_b_text += f"Designated protected with {b4}$ franchise\n"

        return chapter_a_text + chapter_b_text.strip("\n")

    __repr__ = __str__

    def is_protected(self, protection_name: str) -> bool:
        """
        Method to validate if a set of protection include a specific protection.

        protection_name (str): Name of the protection to validate if it includes in the protection set.

        Return:
            A bool.
        """

        protection_status = self.get(protection_name)

        is_protected = False
        if protection_status is None:
            # We use the default is_protected = False
            pass
        elif isinstance(protection_status, str):
            if protection_status == "include":
                is_protected = True
        elif isinstance(protection_status, (int, float)):
            # > 0 since when protected the value is the deductible
            if protection_status > 0:
                is_protected = True

        return is_protected

    def property_damage_protections(self) -> List:
        """
        Method to extract the property damage with a protection (with a deductible). E.g. protection that are
        not by default include under another protection. For example, if a protection is a B1, than B2, B3 and B4
        are include. Thus, this method will return only the B1 protection.

        Return:
            A list of the property damage protection with a deductible.
        """
        property_damage_protections = []
        b1 = self.get("PropertyDamageB1Coverage")
        if b1 is not None and b1 not in (INCLUDE, EXCLUDE):
            property_damage_protections.append("PropertyDamageB1Coverage")

        b2 = self.get("PropertyDamageB2Coverage")
        if b2 is not None and b2 not in (INCLUDE, EXCLUDE):
            property_damage_protections.append("PropertyDamageB2Coverage")

        b3 = self.get("PropertyDamageB3Coverage")
        if b3 is not None and b3 not in (INCLUDE, EXCLUDE):
            property_damage_protections.append("PropertyDamageB3Coverage")

        b4 = self.get("PropertyDamageB4Coverage")
        if b4 is not None and b4 not in (INCLUDE, EXCLUDE):
            property_damage_protections.append("PropertyDamageB4Coverage")

        return property_damage_protections

    def endorsement_protections(self) -> List:
        """
        Method to get a list of all the endorsement protections in CamelCase.

        Return:
            A list of the endorsement.
        """
        return [change_to_camel_case(key) for key, _ in self.__dict__.items() if "f_a_q" in key]

    def get(self, item: str) -> Union[int, None]:
        """
        Method to get a deductible amount for a specific protection (item).

        item (str): Item (the protection) we want the deductible amount.

        Return:
            A int of the deductible amount or a None if not covered.
        """
        try:
            # Attribute are saved in snake case format.
            return getattr(self, change_to_snake_case(item))
        except AttributeError:
            return None



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq20_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 20\n"
        "Travel expenses\n"
        "(Section B)\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer:  \n"
        "Named insured:  \n"
        "Endorsement to automobile insurance policy N:  \n"
        "Effective date: This endorsement will apply from at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Additional insurance premium payable:\n"
        "\t Amount payable:  \n"
        "\t Due date:  \n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement extends coverage under Section B of the insurance contract by replacing "
        "the wording of Article 4.1, ''Travel expenses due to theft of insured vehicle'' with "
        "the wording below.\n"
        "This endorsement will apply only to the specified vehicle and only if the value of "
        "damage to the specified vehicle is greater than the deductible amount applicable "
        "to the loss that caused the damage.\n"
        "''4.1 Travel expenses\n"
        "4.1.1 Description of travel expenses\n"
        "If the named insured is no longer able to use the insured vehicle due to a covered loss, "
        "the insurer will reimburse any expenses incurred for\n"
        "\t leasing of a temporary replacement vehicle;\n"
        "\t public transportation;\n"
        "\t use of taxicab.\n"
        "Upon submission of receipts, payment for the above expenses will be made up to a "
        "maximum of  $ a day and  $ per loss for each insured vehicle.\n"
        "The above amounts cannot be less than the amounts that were specified in "
        "Additional coverages 4.1, of the insurance contract.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "4.1.2 Application of coverage\n"
        "If the entire insured vehicle was stolen, this coverage will apply only to expenses incurred "
        "from 12:01 A.M. the day after the theft is reported to the police or to the insurer.\n"
        "For all other covered losses, this coverage will apply only to expenses incurred:\n"
        "\t from the time at which the insured vehicle can no longer be operated under its own "
        "power due to damage to the vehicle; or\n"
        "\t from the time at which the insured vehicle is delivered for repair, if it can still "
        "be operated in spite of damage to the vehicle.\n"
        "Expenses will be eligible for reimbursement even if the insurance contract has expired "
        "since the loss.\n"
        "Expenses will no longer be eligible for reimbursement once :\n"
        "\t the insured vehicle has been replaced or repaired; or\n"
        "\t a settlement agreement for the loss has been reached before the insured "
        "vehicle is replaced or repaired.''\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq20a_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 20a\n"
        "Travel costs (broad form)\n"
        "(Section B)\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer:  \n"
        "Named insured:  \n"
        "Endorsement to automobile insurance policy N:  \n"
        "Effective date: This endorsement will apply from at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement extends coverage under Section B of the insurance contract by replacing the "
        "wording of Article 4.1, ''Travel costs due to theft of insured vehicle'' with the wording below.\n"
        "This endorsement will apply only to the specified vehicle and only if the value of damage to "
        "the specified vehicle is greater than the deductible amount applicable to the loss that caused the damage.\n"
        "''4.1 Travel costs\n"
        "4.1.1 Description of travel costs\n"
        "If the named insured is no longer able to use the insured vehicle due to a covered loss, "
        "the insurer will reimburse expenses incurred for\n"
        "\t public transportation;\n"
        "\t leasing of a temporary replacement vehicle;\n"
        "\t taxicab fares.\n"
        "Upon submission of receipts, payment for the above expenses will be made up to a maximum of  $ a "
        "day and  $ per loss for each insured vehicle.\n"
        "The above amounts cannot be less than the amounts that were entered in Additional "
        "coverages 4.1, of the insurance contract.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "4.1.2 Application of coverage\n"
        "If the entire insured vehicle was stolen, this coverage will apply only to expenses incurred "
        "from 12:01 A.M. the day after the theft is reported to the police or to the insurer.\n"
        "For all other covered losses, this coverage will apply only to expenses incurred from the "
        "time at which the insured vehicle:\n"
        "\t can no longer be operated under its own power due to damage to the vehicle; or\n"
        "\t is delivered for repair, if it can still be operated in spite of damage to the vehicle.\n"
        "Expenses will be eligible for reimbursement even if the insurance contract has expired since "
        "the loss.\n"
        "Expenses will no longer be eligible for reimbursement once\n"
        "\t the insured vehicle has been replaced or repaired; or\n"
        "\t a settlement agreement for the loss has been reached before the insured vehicle "
        "is replaced or repaired.''\n"
        "4.1.3 Additional costs covered during a trip\n"
        "When a covered loss occurs during a trip, the expenses described in paragraphs "
        "(a) and (b) below will be covered, in addition to the expenses listed in paragraph 4.1.1.\n"
        "The expenses will be covered up to a maximum of  % of the maximum amount payable "
        "per loss shown in paragraph 4.1.1.\n"
        "(a) Any additional travel expenses incurred by the named insured, his or her spouse "
        "or any other person whose domicile is the same as theirs, to:\n"
        "\t continue the trip;\n"
        "\t return to the domicile of the named insured;\n"
        "\t return to where the insured vehicle is usually parked.\n"
        "Additional expenses include the cost of meals and accommodation and the cost "
        "of transportation of personal effects.\n"
        "(b) Any other additional expenses similar to those described in paragraph (a), "
        "incurred to retrieve the insured vehicle where it was repaired and return it to one "
        "of the following places:\n"
        "\t the current location of the named insured, his or her spouse or any "
        "other person whose domicile is the same as theirs. However, if that location is "
        "further away than the travel destination that had been planned before the loss, "
        "only the expenses required to return the insured vehicle to the planned destination "
        "are covered;\n"
        "\t the domicile of the named insured; or\n"
        "\t the place where the insured vehicle is usually parked.\n"
        "The expenses must have been incurred by the named insured, his or her "
        "spouse and any other person whose domicile is the same as theirs or any person of "
        "their choice.''\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq13_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 13c\n"
        "Restriction de la Protection 3 pour les vitres du véhicule\n"
        "(Chapitre B)\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat d'assurance.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N :\n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Cet avenant restreint la Protection 3 du chapitre B du contrat d'assurance en excluant les "
        "dommages occasionnés aux vitres du véhicule visé, sauf s'ils sont occasionnés par :\n"
        "\t l'atterrissage forcé ou la chute d'un aéronef ou d'une partie de cet appareil;\n"
        "\t la crue des eaux;\n"
        "\t l'échouement, la submersion, l'incendie, le déraillement ou la collision de tout véhicule terrestre "
        "ou bateau servant à transporter le véhicule visé;\n"
        "\t les émeutes;\n"
        "\t les explosions;\n"
        "\t la foudre;\n"
        "\t la grêle;\n"
        "\t l'incendie;\n"
        "\t les mouvements populaires;\n"
        "\t les tempêtes de vent;\n"
        "\t les tentatives de vol;\n"
        "\t les tremblements de terre;\n"
        "\t le vol.\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



import random


class SexFaker:
    def __init__(
        self, locale: str, number_of_male_drivers: int = 2860709, population_of_drivers: int = 5546433
    ) -> None:
        """
        Faker to statistically generate a biological gender base on a driver population. By default, it uses Quebec
        province population.

        locale (str): The locale language setting to use for simulation. Can either be `'fr_CA'` or `'en_CA'`.
        number_of_male_drivers (int) : Number of male drivers. Default is Quebec number of male
            driver in 2020 base on the SAAQ bilan routier https://saaq.gouv.qc.ca/saaq/documentation/bilan-routier/.
        population_of_drivers (int): Number of drivers. Default is Quebec number of driver
            in 2020 base on the SAAQ bilan routier https://saaq.gouv.qc.ca/saaq/documentation/bilan-routier/.
        """
        self.number_of_male_drivers = number_of_male_drivers
        self.population_of_drivers = population_of_drivers

        self.proportion_of_male_driver = number_of_male_drivers / population_of_drivers

        if locale == "fr_CA":
            self.gender_1 = "homme"
            self.gender_2 = "femme"
        elif locale == "en_CA":
            self.gender_1 = "male"
            self.gender_2 = "female"
        else:
            raise ValueError(f"The locale {locale} is not supporter. It can either be 'fr_CA' or 'en_CA'.")

    def gender(self) -> str:
        """
        Method to fake a gender.

        Return:
            A gender in the locale language.
        """

        gender = self.gender_1
        u = random.uniform(0, 1)
        if u > self.proportion_of_male_driver:
            gender = self.gender_2
        return gender



def generate_general_pages_header_text(policy_number: str) -> str:
    header_template = (
        "QUEBEC AUTOMOBILE INSURANCE POLICY\n" "Q.P.F. B 1 - OWNER'S FORM\n" f"Policy Number: {policy_number}\n\n"
    )
    return header_template



import random


def association_rebate_faker() -> bool:
    """
    Faker function to fake an association rebate.

    Return:
        A bool if it has an association rebate or not.
    """
    return random.choices([True, False], weights=[0.05, 0.95], k=1)[0]



from unittest import TestCase

import datetime

from risc_generator import Insuree


class InsureeTest(TestCase):
    def setUp(self) -> None:
        self.a_sex = "male"
        self.a_name = "name"
        self.a_date_of_birth = datetime.date(year=2020, month=1, day=1)
        self.an_address = "an address"

    def test_givenNoClaimsNoSuspension_whenCanHaveFAQ41_thenReturnTrue(self):
        insured = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=False,
            number_of_claims_past_years=0,
            association_rebate=True,
        )

        self.assertTrue(insured.can_have_faq_41())

    def test_givenAClaimsNoSuspension_whenCanHaveFAQ41_thenReturnFalse(self):
        insured = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=False,
            number_of_claims_past_years=1,
            association_rebate=True,
        )

        self.assertFalse(insured.can_have_faq_41())

    def test_givenNoClaimsASuspension_whenCanHaveFAQ41_thenReturnFalse(self):
        insured = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=True,
            number_of_claims_past_years=0,
            association_rebate=True,
        )

        self.assertFalse(insured.can_have_faq_41())

    def test_givenAClaimsASuspension_whenCanHaveFAQ41_thenReturnFalse(self):
        insured = Insuree(
            sex=self.a_sex,
            name=self.a_name,
            birth_date=self.a_date_of_birth,
            address=self.an_address,
            suspension=True,
            number_of_claims_past_years=1,
            association_rebate=True,
        )

        self.assertFalse(insured.can_have_faq_41())



import datetime


class Insuree:
    def __init__(
        self,
        sex: str,
        name: str,
        birth_date: datetime.date,
        address: str,
        suspension: bool,
        number_of_claims_past_years: int,
        association_rebate: bool,
    ) -> None:
        self.sex = sex
        self.name = name
        self.birth_date = birth_date
        self.address = address
        self.suspension = suspension
        self.number_of_claims_past_years = number_of_claims_past_years
        self.association_rebate = association_rebate

    def __str__(self) -> str:
        return f"Insured named: {self.name} born the {self.date_of_birth} and living at the address: {self.address}."

    def can_have_faq_41(self) -> bool:
        return self.number_of_claims_past_years == 0 and not self.suspension

    def insuree_is_in_a_rebate_group(self):
        return self.association_rebate

    @property
    def date_of_birth(self) -> str:
        return str(self.birth_date)



# Text generation config

GENERAL_FAKE_PHONE_NUMBER = "123 456-7890"
GENERAL_FAKE_PHONE_NUMBER_1_800 = "1 800 123-4567"
FAKE_FAX_PHONE_NUMBER = "012 345-6789"
FAKE_EMAIL = "service.client@assureur.ca"

CLAIMS_FAKE_PHONE_NUMBER = "098 765-4321"
CLAIMS_FAKE_PHONE_NUMBER_1_800 = "1 800 765-4321"

FAKE_LEGAL_HELP_PHONE_NUMBER = "1 800 012-3456"

FAKE_ROADSIDE_ASSISTANCE_PHONE_NUMBER = "1 877 012-3456"
FAKE_ROADSIDE_PROGRAM_NAME = "ABC"

FAKE_INSURER_ADDRESS = "221 B Baker Street\nVille (Province)\nA1A 1A1"
FAKE_INSURER_WEBSITE = "assureur.ca"

FAKE_INSURER_NAME = "Assureur"

FAKE_FIDELITY_PROGRAM_NAME_FR = "FIDÉLITÉ"
FAKE_FIDELITY_PROGRAM_NAME_EN = "FIDELITY"

FAKE_MENTAL_HEALTH_PROGRAM_NAME = "MentalHealth"


NEW_PAGE_TAG = "<###NEW_PAGE###>"



from risc_generator import Protections, FPQ1Contract, Premium, Vehicle
from risc_generator.content import fr, en
from tests.domain.contract.contract_base import ContractTestBase

INCLUDE = "include"
EXCLUDE = "exclude"


class ContractDeclarationTest(ContractTestBase):
    def test_givenAFRContractNewBoughtCar_whenGenerateItem3Text_thenReturnProperFRText(self):
        a_protections = Protections(protections=self.a_dict_of_protections)
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        a_premium = Premium(**protection_premiums)

        a_vehicle = Vehicle(
            year=self.a_year,
            make=self.a_maker,
            model=self.a_model,
            vin=self.a_vin,
            purchase_condition="neuf",
            electric_vehicle=False,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_3_text(
            formatted_vehicle_complete_details=fpq1_contract.vehicle.format_vehicle_complete_details(
                with_text="Numéro de série"
            ),
            car_ownership=fpq1_contract.car_ownership(),
            vehicle_purchase_condition=fpq1_contract.vehicle.purchase_condition,
            car_with_financing=fpq1_contract.car_with_financing(),
            financing_details=fpq1_contract.financing,
        )

        an_insured_car = "AMaker - AModel - 2020 - Numéro de série 1234567890 - Achat neuf"

        expected = "Article 3. CARACTÉRISTIQUES DU VÉHICULE DÉSIGNÉ 1\n" f"{an_insured_car}\n"

        self.assertEqual(expected, actual)

    def test_givenAFRContractUsedBoughtCar_whenGenerateItem3Text_thenReturnProperFRText(self):
        a_protections = Protections(protections=self.a_dict_of_protections)
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        a_premium = Premium(**protection_premiums)

        a_vehicle = Vehicle(
            year=self.a_year,
            make=self.a_maker,
            model=self.a_model,
            vin=self.a_vin,
            purchase_condition="usagé",
            electric_vehicle=False,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_3_text(
            formatted_vehicle_complete_details=fpq1_contract.vehicle.format_vehicle_complete_details(
                with_text="Numéro de série"
            ),
            car_ownership=fpq1_contract.car_ownership(),
            vehicle_purchase_condition=fpq1_contract.vehicle.purchase_condition,
            car_with_financing=fpq1_contract.car_with_financing(),
            financing_details=fpq1_contract.financing,
        )

        an_insured_car = "AMaker - AModel - 2020 - Numéro de série 1234567890 - Achat usagé"

        expected = "Article 3. CARACTÉRISTIQUES DU VÉHICULE DÉSIGNÉ 1\n" f"{an_insured_car}\n"

        self.assertEqual(expected, actual)

    def test_givenAFRContractNewLoanedCar_whenGenerateItem3Text_thenReturnProperFRText(self):
        a_dict_of_protections = self.a_dict_of_protections.copy()
        a_dict_of_protections.update({"FAQ5ACoverage": INCLUDE})
        a_protections = Protections(protections=a_dict_of_protections)
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        a_premium = Premium(**protection_premiums)

        a_vehicle = Vehicle(
            year=self.a_year,
            make=self.a_maker,
            model=self.a_model,
            vin=self.a_vin,
            purchase_condition="neuf",
            electric_vehicle=False,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_3_text(
            formatted_vehicle_complete_details=fpq1_contract.vehicle.format_vehicle_complete_details(
                with_text="Numéro de série"
            ),
            car_ownership=fpq1_contract.car_ownership(),
            vehicle_purchase_condition=fpq1_contract.vehicle.purchase_condition,
            car_with_financing=fpq1_contract.car_with_financing(),
            financing_details=fpq1_contract.financing,
        )

        an_insured_car = "AMaker - AModel - 2020 - Numéro de série 1234567890 - Location neuf"
        a_creditor_details = "A Bank An Address for The Bank"

        expected = (
            "Article 3. CARACTÉRISTIQUES DU VÉHICULE DÉSIGNÉ 1\n"
            f"{an_insured_car}\n"
            "Locateur qui a droit aux indemnités du chapitre B, selon son intérêt - Sujet à l'avenant F.A.Q. N 5a\n"
            f"{a_creditor_details}\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAFRContractNewCreditorCar_whenGenerateItem3Text_thenReturnProperFRText(self):
        a_dict_of_protections = self.a_dict_of_protections.copy()
        a_dict_of_protections.update({"FAQ23ACoverage": INCLUDE})
        a_protections = Protections(protections=a_dict_of_protections)
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        a_premium = Premium(**protection_premiums)

        a_vehicle = Vehicle(
            year=self.a_year,
            make=self.a_maker,
            model=self.a_model,
            vin=self.a_vin,
            purchase_condition="neuf",
            electric_vehicle=False,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual = fr.declaration_articles.generate_item_3_text(
            formatted_vehicle_complete_details=fpq1_contract.vehicle.format_vehicle_complete_details(
                with_text="Numéro de série"
            ),
            car_ownership=fpq1_contract.car_ownership(),
            vehicle_purchase_condition=fpq1_contract.vehicle.purchase_condition,
            car_with_financing=fpq1_contract.car_with_financing(),
            financing_details=fpq1_contract.financing,
        )

        an_insured_car = "AMaker - AModel - 2020 - Numéro de série 1234567890 - Achat neuf"
        a_creditor_details = "A Bank An Address for The Bank"

        expected = (
            "Article 3. CARACTÉRISTIQUES DU VÉHICULE DÉSIGNÉ 1\n"
            f"{an_insured_car}\n"
            "Créancier qui a droit aux indemnités du chapitre B, selon son intérêt - Sujet à l'avenant F.A.Q. N 23a\n"
            f"{a_creditor_details}\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractNewBoughtCar_whenGenerateItem3Text_thenReturnProperENText(self):
        a_protections = Protections(protections=self.a_dict_of_protections)
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        a_premium = Premium(**protection_premiums)

        a_vehicle = Vehicle(
            year=self.a_year,
            make=self.a_maker,
            model=self.a_model,
            vin=self.a_vin,
            purchase_condition="new",
            electric_vehicle=False,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_3_text(
            formatted_vehicle_complete_details=fpq1_contract.vehicle.format_vehicle_complete_details(
                with_text="Serial number"
            ),
            car_ownership=fpq1_contract.car_ownership(),
            vehicle_purchase_condition=fpq1_contract.vehicle.purchase_condition,
            car_with_financing=fpq1_contract.car_with_financing(),
            financing_details=fpq1_contract.financing,
        )

        an_insured_car = "AMaker - AModel - 2020 - Serial number 1234567890 - Purchase new"

        expected = "Item 3. PARTICULARS OF THE DESCRIBED VEHICLE 1\n" f"{an_insured_car}\n"

        self.assertEqual(expected, actual)

    def test_givenAENContractUsedBoughtCar_whenGenerateItem3Text_thenReturnProperENText(self):
        a_protections = Protections(protections=self.a_dict_of_protections)
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        a_premium = Premium(**protection_premiums)

        a_vehicle = Vehicle(
            year=self.a_year,
            make=self.a_maker,
            model=self.a_model,
            vin=self.a_vin,
            purchase_condition="used",
            electric_vehicle=False,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_3_text(
            formatted_vehicle_complete_details=fpq1_contract.vehicle.format_vehicle_complete_details(
                with_text="Serial number"
            ),
            car_ownership=fpq1_contract.car_ownership(),
            vehicle_purchase_condition=fpq1_contract.vehicle.purchase_condition,
            car_with_financing=fpq1_contract.car_with_financing(),
            financing_details=fpq1_contract.financing,
        )

        an_insured_car = "AMaker - AModel - 2020 - Serial number 1234567890 - Purchase used"

        expected = "Item 3. PARTICULARS OF THE DESCRIBED VEHICLE 1\n" f"{an_insured_car}\n"

        self.assertEqual(expected, actual)

    def test_givenAENContractNewLoanedCar_whenGenerateItem3Text_thenReturnProperENText(self):
        a_dict_of_protections = self.a_dict_of_protections.copy()
        a_dict_of_protections.update({"FAQ5ACoverage": INCLUDE})
        a_protections = Protections(protections=a_dict_of_protections)
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        a_premium = Premium(**protection_premiums)

        a_vehicle = Vehicle(
            year=self.a_year,
            make=self.a_maker,
            model=self.a_model,
            vin=self.a_vin,
            purchase_condition="new",
            electric_vehicle=False,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_3_text(
            formatted_vehicle_complete_details=fpq1_contract.vehicle.format_vehicle_complete_details(
                with_text="Serial number"
            ),
            car_ownership=fpq1_contract.car_ownership(),
            vehicle_purchase_condition=fpq1_contract.vehicle.purchase_condition,
            car_with_financing=fpq1_contract.car_with_financing(),
            financing_details=fpq1_contract.financing,
        )

        an_insured_car = "AMaker - AModel - 2020 - Serial number 1234567890 - Rental new"
        a_creditor_details = "A Bank An Address for The Bank"

        expected = (
            "Item 3. PARTICULARS OF THE DESCRIBED VEHICLE 1\n"
            f"{an_insured_car}\n"
            "Lessor entitled to Chapter B benefits, according to his interest - Subject to the "
            "Q.E.F. endorsement N 5a\n"
            f"{a_creditor_details}\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContractNewCreditorCar_whenGenerateItem3Text_thenReturnProperENText(self):
        a_dict_of_protections = self.a_dict_of_protections.copy()
        a_dict_of_protections.update({"FAQ23ACoverage": INCLUDE})
        a_protections = Protections(protections=a_dict_of_protections)
        a_premium_amount = 1
        protection_premiums = {
            "LiabilityPremium": a_premium_amount,
            "PropertyDamageB2CoveragePremium": a_premium_amount,
            "PropertyDamageB3CoveragePremium": a_premium_amount,
            "FAQ20ACoveragePremium": a_premium_amount,
            "FAQ33CoveragePremium": a_premium_amount,
            "FAQ34CoveragePremium": a_premium_amount,
            "FAQ43CoveragePremium": a_premium_amount,
        }

        a_premium = Premium(**protection_premiums)

        a_vehicle = Vehicle(
            year=self.a_year,
            make=self.a_maker,
            model=self.a_model,
            vin=self.a_vin,
            purchase_condition="new",
            electric_vehicle=False,
        )

        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=a_protections,
            premium=a_premium,
            financing=self.a_creditor,
            locale=self.a_en_locale,
        )

        actual = en.declaration_articles.generate_item_3_text(
            formatted_vehicle_complete_details=fpq1_contract.vehicle.format_vehicle_complete_details(
                with_text="Serial number"
            ),
            car_ownership=fpq1_contract.car_ownership(),
            vehicle_purchase_condition=fpq1_contract.vehicle.purchase_condition,
            car_with_financing=fpq1_contract.car_with_financing(),
            financing_details=fpq1_contract.financing,
        )

        an_insured_car = "AMaker - AModel - 2020 - Serial number 1234567890 - Purchase new"
        a_creditor_details = "A Bank An Address for The Bank"

        expected = (
            "Item 3. PARTICULARS OF THE DESCRIBED VEHICLE 1\n"
            f"{an_insured_car}\n"
            "Creditor entitled to Chapter B benefits, according to his interest - Subject to the "
            "Q.E.F. endorsement N 23a\n"
            f"{a_creditor_details}\n"
        )

        self.assertEqual(expected, actual)
